/*   
ZZObs.java
 *    
 *    Copyright (c) 2000, Ted Nelson and Tuomas Lukka
 *
 *    You may use and distribute under the terms of either the GNU Lesser
 *    General Public License, either version 2 of the license or,
 *    at your choice, any later version. Alternatively, you may use and
 *    distribute under the terms of the XPL.
 *
 *    See the LICENSE.lgpl and LICENSE.xpl files for the specific terms of 
 *    the licenses.
 *
 *    This software is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the README
 *    file for more details.
 *
 */
/*
 * Written by Tuomas Lukka
 */

package org.gzigzag;
import java.util.*;

/** A class used to trigger ZZObses.
 */

public class ZZObsTrigger {
String rcsid = "$Id: ZZObsTrigger.java,v 1.3 2000/09/19 10:31:58 ajk Exp $";

    // XXX Store weak references, not hard ones!
    static Vector allTrigs = new Vector();

    ZZObsTrigger() {
	allTrigs.addElement(this);
    }

    /** An object which keeps track of the observations of a single
     * ZZObs.
     */
    class Gob {
	/** Linked list of Gobs, plus list of all objects that this
	 * Gob observes.
	 * This is a tricky part and this is also what makes us blazingly
	 * fast. And difficult to explain. 
	 * <p>
	 * First of all, there is a key for every object that this
	 * gob's <code>obs</code> has been <code>addObs</code>'ed for.
	 * The value depends on whether there are other gobs observing
	 * the same thing. If there are, then the hashtables form a linked
	 * list. If not, or if this is the last element of the linked 
	 * list, the value in the Hashtable is <strong>this gob</strong>.
	 * That may seem odd until you realize that 1) hashtables can't
	 * store nulls, and 2) <code>this</code> is a pointer that is
	 * very likely to be in a register so fast to compare with
	 * the reference.
	 */
	Hashtable next = new Hashtable();
	ZZObs obs;
    }

    /** Key: ZZObs; Value: Gob. */
    Hashtable gobs = new Hashtable();

    /** Key: Object (trigger); Value: Gob. */
    Hashtable trigs = new Hashtable();

    
    /** Called when something is changed.
     * This call removes instances of the ZZObses triggered from
     * <em>all</em> existing ZZObsTrigger objects.
     */
    public synchronized void chg(Object o) {
	Gob g;
	while((g = (Gob)trigs.get(o)) !=null) {
	    // Better to remove first, just in case 
	    for(int i=0; i<allTrigs.size(); i++) {
		((ZZObsTrigger)allTrigs.elementAt(i)).rmObs(g.obs); 
		    // Doesn't clear the field.
	    }

	    // XXX Should do now or queue to ZZUpdateManager?
	    g.obs.chg();
	}
	ZZUpdateManager.chg();
    }

    /** Called to add an observer.
     */
    public synchronized void addObs(Object o, ZZObs obs) {
	Gob g = (Gob) gobs.get(obs);
	if(g==null) {
	    gobs.put(obs, g = new Gob());
	    g.obs = obs;
	}
	if(g.next.get(o) != null) return; // Already observing.
	Object t = trigs.get(o);
	trigs.put(o, g);
	g.next.put(o, (t==null ? g : t));
    }

    /** Called to remove observers.
     */
    public synchronized void rmObs(ZZObs obs) {
	Gob g = (Gob)gobs.get(obs);
	if(g == null) return; // Wasn't observing anything
	gobs.remove(obs);
	for(Enumeration e = g.next.keys(); 
	    e.hasMoreElements(); ) {
	    Object o = e.nextElement();
	    Object v = g.next.get(o);
	    if(v == g)
		trigs.remove(o);
	    else
		trigs.put(o, v);

	}
    }
}




