/*   
ZZWindows.java
 *    
 *    Copyright (c) 2000, Ted Nelson and Tuomas Lukka
 *
 *    You may use and distribute under the terms of either the GNU Lesser
 *    General Public License, either version 2 of the license or,
 *    at your choice, any later version. Alternatively, you may use and
 *    distribute under the terms of the XPL.
 *
 *    See the LICENSE.lgpl and LICENSE.xpl files for the specific terms of 
 *    the licenses.
 *
 *    This software is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the README
 *    file for more details.
 *
 */
/*
 * Written by Tuomas Lukka, killing by Antti-Juhani Kaijanaho
 */

package org.gzigzag;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/** XXX NOT FINISHED. This is where the placement of windows is handled.
 * All windows that are created are ZZCanvasViews.
 * We use three dimensions: d.1 and d.2 for the widget tree 
 * and the list of
 * <p>
 * Only creates (and updates) toplevel windows if toplevels is set, to
 * provide for non-toplevel environments like applets.
 */

public class ZZWindows {
public static final String rcsid = "$Id: ZZWindows.java,v 1.21 2000/12/11 09:03:03 raulir Exp $";
    public static boolean dbg = true; 
    static final void p(String s) { if(dbg) System.out.println(s); }

    static boolean toplevels = true;

    static Object sync = new Object();

    static Hashtable wins = new Hashtable();
    static Hashtable ex;

    static Hashtable views = new Hashtable();
    static Component getContent(ZZCell c) {
	synchronized(sync) {
	    String txt = c.getText();
	    // return null;
	    if(txt.equals("Canvas")) {
		ZZCanvasView v;
		if(wins.get(c) instanceof ZZCanvasView)
		    v = (ZZCanvasView) wins.get(c);
		else 
		    wins.put(c, v = new ZZCanvasView());
		ZZCell vcell = c.s("d.1", 1);
		if(vcell == null)
		    throw new ZZError("No viewcell given!");
		v.setViewcell(vcell);
		return v;
	    } else if(txt.equals("Split")) {
	    } else if(txt.equals("Text")) {
	    }
	    throw new ZZError("Invalid window type '"+txt+"'");
	}
    }

    // DON'T access these outside here yet! Not compatible.
    static Rectangle bounds(ZZCell wid) {
	synchronized(sync) {
	    int[] c = ZZUtil.getInts(wid, "d.bounds", false, 4, 4);
	    return new Rectangle(c[0], c[1], c[2], c[3]);
	}
    }
    static public void setBounds(ZZCell wid, Rectangle r) {
	synchronized(sync) {
	    ZZUtil.putInts(wid, "d.bounds", false, 
		new int[] { r.x, r.y, r.width, r.height });
	}
    }

    static public void update(ZZSpace s) {
	create(startCell(s));
    }

    static public void newToplevelCanvas(ZZSpace s, String id, 
	    Rectangle bounds, 
	    Color bg, ZZCell viewCell) {
	synchronized(sync) {
	    ZZCell start = startCell(s);
	    ZZCell l = start.getOrNewCell("d.1", 1);
	    // This is the main window cell.
	    l = l.N("d.2", 1);
	    if(bg != null)
		l.N("d.color",1).setText(String.valueOf(bg.getRGB()));

	    l.setText(id);
	    setBounds(l, bounds);
	    l.insert("d.1", 1, viewCell);
	    l.N("d.1", 1).setText("Canvas");

	    update(s);
	}
    }

    static public void killToplevel(ZZCell wid) {
        synchronized (sync) {
            ZZSpace space = wid.getSpace();
            // XXX what about window parameters?
            wid.delete();
            update(space);
        }
    }

    static ZZCell startCell(ZZSpace s) {
	ZZCell start = ZZDefaultSpace.findOnClientlist(s, "Windows", true);
	return start;
    }

    static void create(ZZCell start) {
	if(!toplevels) return;
		
	synchronized(sync) {
	    ex = new Hashtable();
	    for(ZZCell topl = start.getOrNewCell("d.1", 1).s("d.2", 1);
		topl != null;
		topl = topl.s("d.2", 1)) {

		p("Start next");
		Frame f = (Frame)wins.get(topl);
		if(f == null) {
		    wins.put(topl, f = new Frame());
		    final ZZCell wc = topl;
		    final Component w = f;

		    ScalableFont.fmComp=f;
		    // XXX Causing an update cycle???
		    f.addComponentListener(new ComponentAdapter() {
			public void componentMoved(ComponentEvent e) {
			    p("ComponentMoved: "+w.getBounds());
			    setBounds(wc, w.getBounds());
			}
			public void componentResized(ComponentEvent e) {
			    p("ComponentResized: "+w.getBounds());
			    setBounds(wc, w.getBounds());
			}
		    });
                    f.addWindowListener(new WindowAdapter() {
                            public void windowClosing(WindowEvent e) {
                                p("WindowClosing");
                                killToplevel(wc);
                            }
                        });
                }
		ex.put(topl, topl);
		f.setTitle(topl.getText());

		p("Set bounds: "+bounds(topl));
		f.setBounds(bounds(topl));

		ZZCell col;
		if((col=topl.s("d.color", 1)) != null) {
		    String colText = col.getText();
		    // See if it looks like an URL or a number.
		    if(Character.isDigit(colText.charAt(0)) ||
		       colText.charAt(0) == '-') 
			f.setBackground( Color.decode(colText) );
		    else {
			ZZLogger.log("Not digit first! '"+colText+"'");
			// XXX Find image, store it in the right place (should be 
			// in content...
			/*
			URL url = null;
			try {
			    url = new URL(cursor.h("d.photo", 1).getText());
			} catch(MalformedURLException e) {
			    String s= e.toString();
			    g.drawString(s, 0, 0);
			    return;
			}
			if(!url.equals(cur)) {
			    cur = url;
			    img = getToolkit().getImage(url);
			}
			if(!g.drawImage(img, 0, 0, null)) {
			    repaint(1000);
			}
			*/
		    }

		    if((col=col.s("d.color", 1)) != null) 
			f.setForeground( Color.decode(col.getText()) );
		}

		// Content
		p("Creating "+topl.getText());
		ZZCell cont = topl.s("d.1", 1);
		Component content = getContent(cont);
		f.add(content);
		f.show();
		f.repaint();

		p("Created "+topl.getText());
		ZZUpdateManager.chg();
	    }
            for (Enumeration e = wins.keys(); e.hasMoreElements();) {
                ZZCell cell = (ZZCell) e.nextElement();
                Object o = wins.get(cell);
                if (!(o instanceof Frame)) continue;
                Frame frame = (Frame) o;
                if (!ex.containsKey(cell)) {
                    p("disposing frame");
                    frame.dispose();
                    wins.remove(cell);
                }
            }
	}
    }

}
