/*   
Data.java
 *    
 *    Copyright (c) 2000, Ted Nelson and Tuomas Lukka
 *
 *    You may use and distribute under the terms of either the GNU Lesser
 *    General Public License, either version 2 of the license or,
 *    at your choice, any later version. Alternatively, you may use and
 *    distribute under the terms of the XPL.
 *
 *    See the LICENSE.lgpl and LICENSE.xpl files for the specific terms of 
 *    the licenses.
 *
 *    This software is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the README
 *    file for more details.
 *
 */
/*
 * Written by Benjamin Fallenstein
 */
package org.gzigzag.flowing;
import org.gzigzag.*;

/** An immutable list of Flowing Clang data pieces.
 */

public final class Data {
String rcsid = "$Id: Data.java,v 1.1 2000/10/13 13:52:24 bfallenstein Exp $";

    private Object[] obs;

    public Data() { obs = new Object[0]; }
    public Data(Object o1) { obs = new Object[]{o1}; }
    public Data(Object o1, Object o2) {
	obs = new Object[]{o1,o2}; }
    public Data(Object o1, Object o2, Object o3) { 
	obs = new Object[]{o1,o2,o3}; }
    public Data(Object o1, Object o2, Object o3, Object o4) { 
	obs = new Object[]{o1,o2,o3,o4}; }
    public Data(Object o1, Object o2, Object o3, Object o4, Object o5) { 
	obs = new Object[]{o1,o2,o3,o4,o5}; }

    public Data(Object[] o) {
	obs = new Object[o.length];
	System.arraycopy(o, 0, obs, 0, o.length);
    }

    public int len() { return obs.length; }
    
    public Object o(int at) { return obs[at]; }
    public ZZCell c(int at) {
	if(!(obs[at] instanceof ZZCell))
	    throw new ZZError("Wrong flowing clang datum "+at+": not a cell");
	return (ZZCell)obs[at];
    }
    public String s(int at) {
	Object o = obs[at];
	if(o instanceof String)
	    return (String)o;
	else if(o instanceof Integer)
	    return ""+((Integer)o).intValue();
	else if(o instanceof ZZCell)
	    return ((ZZCell)o).getText();
	else if(o instanceof Boolean) {
	    if(((Boolean)o).booleanValue())
		return "true";
	    else
		return "false";
	} else
	    throw new ZZError("Wrong flowing clang datum "+at+": not a str");
    }
    public boolean b(int at) {
	Object o = obs[at];
	if(o instanceof Boolean)
	    return ((Boolean)o).booleanValue();
	else {
	    String str = s(at);
	    if(str.equals("true"))
		return true;
	    else if(str.equals("false"))
		return false;
	    else
	        throw new ZZError("Wrong flowing clang datum " + at + ": " +
			          "not a boolean");
	}
    }
    public int i(int at) {
	Object o = obs[at];
	if(o instanceof Integer)
	    return ((Integer)o).intValue();
	else {
	    try {
		return Integer.parseInt(s(at));
	    } catch(NumberFormatException e) {
		throw new ZZError("Wrong flowing clang datum " + at + ": " +
				  "not an integer");
	    }
	}
    }
}
