/*   
DirSet.java
 *    
 *    Copyright (c) 2000, Ted Nelson and Tuomas Lukka
 *
 *    You may use and distribute under the terms of either the GNU Lesser
 *    General Public License, either version 2 of the license or,
 *    at your choice, any later version. Alternatively, you may use and
 *    distribute under the terms of the XPL.
 *
 *    See the LICENSE.lgpl and LICENSE.xpl files for the specific terms of 
 *    the licenses.
 *
 *    This software is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the README
 *    file for more details.
 *
 */
/*
 * Written by Tuomas Lukka, locking by Antti-Juhani Kaijanaho
 */
package org.gzigzag;
import java.util.*;
import java.io.*;

/** A directory implementation of WritableSet
 */

// Beginning of block: 
// long: index of next block
// long: offset in subfile0 to the name.

public class DirSet implements WritableSet, SafeExit.Cleanupable {
public static final String rcsid = "$Id: DirSet.java,v 1.9 2000/11/03 08:01:05 ajk Exp $";
    File dir;
    String mode;

    final String lockfilename = "LOCK";

    public String[] listWritables() { return dir.list(); }

    public class LockedException extends ZZError {
        LockedException() {
            super("DirSet is locked");
        }
    }

    public Writable getWritable(String id) {
	if(id == null || id.equals(""))
	    throw new ZZError("Can't use writables with null or empty name");
	try {
	    File fn = new File(dir, id);
	    final RandomAccessFile f = new RandomAccessFile(fn, mode);
	    return new FileWritable(f) ;
	} catch(Exception e) {
	    ZZLogger.exc(e);
	    throw new ZZError(" "+e);
	}
    }

    public boolean exists(String id) {
	File fn = new File(dir, id);
	return fn.exists();
    }

    public boolean locked() {
        File fn = new File(dir, lockfilename);
        return fn.exists();
    }

    public void lock() {
        System.err.println("locking disabled");
//          if (locked()) throw new LockedException();
//          try {
//              File fn = new File(dir, lockfilename);
//              RandomAccessFile f = new RandomAccessFile(fn, "rw");
//          } catch(Exception e) {
//              e.printStackTrace();
//              throw new ZZError(" " + e);
//          }
    }

    public void unlock() {
//          System.err.println("trying to unlock");
//          if (!locked()) {
//              System.err.println("not locked");
//              return;
//          }
//          File fn = new File(dir, lockfilename);
//          try {
//          System.err.println(fn.getCanonicalPath());
//          } catch (Exception e) { System.err.println(""+e); }
//          if (fn.delete()) {
//              System.err.println("unlocked");
//          } else {
//              System.err.println("failed to unlock");
//          }
    }

    public DirSet(File dir0, String mod, boolean forcelock) {
	try {
	    mode = mod;
	    dir = dir0;
	    if(dir.exists() && !dir.isDirectory()) {
		throw new ZZError("MUST BE DIRECTORY!!!! "+dir);
	    }
	    if(!dir.exists()) {
		dir.mkdirs();
	    }
	} catch(Exception e) {
	    ZZLogger.exc(e);
	    throw new ZZError(" "+e);
	}
        if (locked() && forcelock) unlock();
        if (locked()) throw new LockedException();
        SafeExit.registerObject(this);
        lock();
    }

    public void cleanup() throws Throwable {
        unlock();
    }

    protected void finalize() throws Throwable {
        try {
            unlock();
        } finally {
            super.finalize();
        }
    }

    public DirSet(File dir0, String mod) {
        this(dir0, mod, false);
    }

}


