/*   
WritableHash.java
 *    
 *    Copyright (c) 2000, Ted Nelson and Tuomas Lukka
 *
 *    You may use and distribute under the terms of either the GNU Lesser
 *    General Public License, either version 2 of the license or,
 *    at your choice, any later version. Alternatively, you may use and
 *    distribute under the terms of the XPL.
 *
 *    See the LICENSE.lgpl and LICENSE.xpl files for the specific terms of 
 *    the licenses.
 *
 *    This software is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the README
 *    file for more details.
 *
 */
/*
 * Written by Tuomas Lukka
 */
package org.gzigzag;
import java.util.*;
import java.io.*;

/** DEPRECATED. A class which keeps track of an append-only hash-like thing.
 * Hash is a misnomer:
 * The actual data structure is a tree and not necessarily very
 * efficient. XXX NOT ANY MORE - HASH, EXPLAIN HERE.
 * <p>
 * subclassing can be used to make this more efficient.
     * Records with values should be made by deriving from TreeRecord. 
     * Keeps track of the previous cells in the stream with a lesser
     * and greater hash value than this one's, which allows searching
     * in a logarithmic time (well, linear worst-case)
 */

public abstract class WritableHash {
public static final String rcsid = "$Id: WritableHash.java,v 1.8 2000/11/03 08:01:05 ajk Exp $";
    static public boolean dbg = false;
    static final void p(String s) { if(dbg) System.out.println(s); }

    ByteArrayOutputStream bos = new ByteArrayOutputStream();
    DataOutputStream os = new DataOutputStream(bos);
    WritableIS wis;
    DataInputStream dis;

    long reads;
    Writable w;
    Recs.HdrRecord hdr;

    WritableHash(Writable writable) {
	w = writable;
	dis = new DataInputStream(wis = new WritableIS(w));
    }
    public void start() {
	hdr = readHdrRecord();
    }
    public static class InvalidHeader extends ZZError {
        public InvalidHeader(String s) { super(s); }
    }
    public Recs.HdrRecord readHdrRecord() {
        Recs.Record r = readRecord(0);
        if (!(r instanceof Recs.HdrRecord))
            throw new InvalidHeader("Invalid header");
       return (Recs.HdrRecord)r;
    }
    public void clearStart() {
	putRecord(0, createnewHdrRecord());
	start();
	rehash((short)13);
    }
    
    public void addBytes(long n) {
	hdr.end += n;
	putRecord(0, hdr);
    }

    public void rehash(short bits) {
    p("REHASH!");
    try {
	if(bits > 32) return;
	short obits = hdr.nbits;
	long ohash = hdr.lasthash;
	if(bits <= obits)
	    throw new ZZError("Can't rehash to smaller!");
	Recs.HashRecord r = new Recs.HashRecord();
	r.nbits = bits;
	putRecord(hdr.end, r);

	addBytes(r.length());

	long hoff = hdr.end;
	hdr.lasthash = hoff;

	int nent = (1<<bits);
	addBytes(8*nent);


	hdr.nbits = bits;
	hdr.mask = 0;
	int m = 1;
	for(int i=0; i<bits; i++)  {
	    hdr.mask |= m; m <<= 1;
	}
	putRecord(0, hdr);

	bos.reset();

	for(int i=0; i<nent; i++) {
	    if(ohash < 0) {
		os.writeLong(-1);
	    } else {
		throw new ZZError("No real rehash yet!");
	    }
	}

	w.write(hoff, bos.toByteArray());

    } catch(Exception e) {
	ZZLogger.exc(e);
	throw new ZZError("IO ERROR");
    }
    }

    public Recs.HdrRecord createnewHdrRecord() {
	Recs.HdrRecord r = newHdrRecord();
	fillHdrRecord(r);
	return r;
    }
    public Recs.HdrRecord newHdrRecord() {
	return new Recs.HdrRecord();
    }

    public void fillHdrRecord(Recs.HdrRecord h) {
	h.nbits = -1;
	h.lasthash = -1;
	h.end = h.length();
    }

    


// get the right type of hdr, filled out with correct values for a new file.


    // CACHE!!!
    public Recs.Record readRecord(long offs) {
	if(offs < 0) return null;
	wis.seek(offs);
	Recs.Record r = Recs.readRecord(offs, dis);
	p("ReadRec: "+offs+"\n "+r);
	reads++;
	return r;
    }

    public Recs.TreeRecord findRecord(Recs.TreeRecord before, int hash) {
    try {
	Recs.TreeRecord  r = null;
	if(before==null) {
	    // Look up from hash
	    long ho = hashOffset(hash);
	    wis.seek(ho);
	    long rec = dis.readLong();
	    r = (Recs.TreeRecord)readRecord(rec);
	    while(r!=null && r.hash != hash)
		r = (Recs.TreeRecord)readRecord(r.prevwhash);
	} else {
	    // Follow linked list
	    r = before;
	    do {
		r = (Recs.TreeRecord)readRecord(r.prevwhash);
	    } while(r!=null && r.hash != hash);
	}
	return r;
    } catch(Exception e) {
	ZZLogger.exc(e);
	throw new ZZError("IO");
    }
    }

    final long hashOffset(int hash) {
	if(hdr.lasthash < 0) throw new ZZError("NO HASH!");
	return hdr.lasthash + (hash & hdr.mask) * 8;
    }

    // XXX Note that dimhash uses the hdr.end field BETWEEN calls to get
    // offset of next!!!
    // Trouble if hash expanded BEFORE putting record - must expand AFTER.
    public long putTreeRecord(Recs.TreeRecord r) {
    try {
	long o = hdr.end;
	    // p("PTR: "+hdr);
	    // XXX
	long ho = hashOffset(r.hash); 
    
    // maintain linked list of records.
	wis.seek(ho);
	long prev = dis.readLong();

	r.prevwhash = (int)prev;
	putRecord(o, r);
	addBytes(r.length());

	bos.reset();
	os.writeLong(o);
	w.write(ho, bos.toByteArray());
	w.flush();
	return o;
    } catch(Exception e) {
	ZZLogger.exc(e);
	throw new ZZError("IO");
    }
    }

    public void putRecord(long o, Recs.Record r) {
	// p("Putrecord: "+o+"\n "+r);
	r.offset = o;
	bos.reset();
	Recs.writeRecord(r, os); 
	byte[]b = bos.toByteArray(); 
	w.write(o, b);

    }
}
