/*   
ZZPersistentDimSpace.java
 *    
 *    Copyright (c) 2000, Ted Nelson and Tuomas Lukka
 *
 *    You may use and distribute under the terms of either the GNU Lesser
 *    General Public License, either version 2 of the license or,
 *    at your choice, any later version. Alternatively, you may use and
 *    distribute under the terms of the XPL.
 *
 *    See the LICENSE.lgpl and LICENSE.xpl files for the specific terms of 
 *    the licenses.
 *
 *    This software is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the README
 *    file for more details.
 *
 */
/*
 * Written by Tuomas Lukka
 */
package org.gzigzag;
import java.util.*;

/** DEPRECATED: A dimspace that uses a WritableSet to be persistent.
 * This is on the way out in favor of the new format in ZZCacheDimSpace.
 * XXX Namespace!
 */

public class ZZPersistentDimSpace extends ZZDimSpace {
public static final String rcsid = "$Id: ZZPersistentDimSpace.java,v 1.13 2000/11/30 08:44:32 ajk Exp $";
    public static boolean dbg = false;
    final static void p(String s) { if(dbg) System.out.println(s); }
    final static void pa(String s) { System.out.println(s); }

    public ZZPersistentDimSpace(WritableSet w) {
	this(w, false);
    }
    public ZZPersistentDimSpace(WritableSet w, boolean usecache) {
	wrs = w;
	boolean ex = wrs.exists("CONTENT");
	ct =  new ContentHash(wrs.getWritable("CONTENT"));
	p("CONTENT: "+ex);
	if(ex)
	    ct.start();
	else
	    ct.clearStart();
	root = wrs.getWritable("ROOT");
	if(ex) {
	    rootrec = (Recs.RootRecord)Recs.readRecord(0, root);
	    nextID = ""+rootrec.nextCell;
	} else {
	    rootrec = new Recs.RootRecord();
	    try {
	        rootrec.nextCell = Long.parseLong(nextID);
	        Recs.writeRecord(rootrec, root);
	    } catch(NumberFormatException e) { };
	}
    }
    protected String getFreeCellID() {
	String curID=super.getFreeCellID();
	try {
		rootrec.nextCell = Long.parseLong(nextID);
		Recs.writeRecord(rootrec, root);
	} catch(NumberFormatException e) { };
	return curID;
    }

    WritableSet wrs;
    Writable root;
    Recs.RootRecord rootrec;
    ContentHash ct;


    protected Span getSpan(String id) {
	Object o = ct.get(id); 
	if(!(o instanceof Span))
	    return null;
	return (Span)o;
    }
    protected String getText(String id) {
	Object o = ct.get(id); 
	if(o instanceof Span) 
	    o = ((Span)o).getString(getHomeCell());
	return (String)o;
    }

    protected void setText(String id, Object cont) {
	ct.put(id, cont);
    }

    protected ZZDimension createDimension(String s) {
	if(!validDim(s)) return null;
	boolean ex = wrs.exists(s);
	Writable wr = wrs.getWritable(s);
	p("CREATEDIM: "+s+" ex: "+ex);

        ZZDimension d;
        if(s.equals("d.cellcreation")) {
                d = new IDDimension();
        } else {
                d = new ZZPersistentDimension(wr, ex);
        }

	d.setSpace(this);
	return d;
    }

    Hashtable scrolls = new Hashtable();

    public StringScroll getStringScroll() {
	return getStringScroll("text");
    }
    public StringScroll getStringScroll(String name) {
	StringScroll scr;
	if((scr = (StringScroll)scrolls.get(name)) == null) {
	    Writable wr = wrs.getWritable("STRSCR_"+name);
	    scr = new StringScroll(name, wr, false);
	    // XXX ???
	    Scroll.register(name, scr);
	    scrolls.put(name, scr);
	}
	return scr;
    }

    public void recreateMasterDimList() {
        ZZCellHandle home = (ZZCellHandle)getHomeCell();
        ZZCellHandle testcell1 = (ZZCellHandle)newCell();
        ZZCellHandle testcell2 = (ZZCellHandle)newCell();
        Vector v = new Vector();
        try {
            String[] writables = wrs.listWritables();
            for (int i = 0; i < writables.length; i++) {
                if (writables[i].equals("CONTENT")
                    || writables[i].equals("ROOT")) {
                    p("Ignoring " + writables[i]);
                    continue;
                }
                p("TRYING: " + writables[i]);
                Writable w = wrs.getWritable(writables[i]);
                try {
                    ZZPersistentDimension dim =
                        new ZZPersistentDimension(w, true);
                    // test moving in the dimension
                    dim.s(home, 1, null);
                    // test creating connections
                    dim.connect(testcell1, testcell2);
                    // test moving in the dimension again
                    dim.s(testcell1, 1, null);
                } catch (ZZInvalidDimensionError e) {
                    p("FAILED: " + writables[i]);
                    continue;
                }
                p("SUCCESS: " + writables[i]);
                v.addElement(writables[i]);
            }
        } finally {
            testcell1.delete();
            testcell2.delete();
        }
        ZZCell list = home.N();
        for (int i = 0; i < v.size(); i++) {
            ZZCell c = list.N(d.masterdim, 1);
            c.setText((String) v.elementAt(i));
            pa("Added dimension " + c.getText() + " to master dimension list");
        }
        ZZCell lstart = list.s(d.masterdim, 1);
        list.delete();
        home.disconnect(d.masterdim, 1);
        home.connect(d.masterdim, lstart);
    }


}
