/*   
Nile2Ops.java
 *    
 *    Copyright (c) 2000, Ted Nelson and Tuomas Lukka
 *
 *    You may use and distribute under the terms of either the GNU Lesser
 *    General Public License, either version 2 of the license or,
 *    at your choice, any later version. Alternatively, you may use and
 *    distribute under the terms of the XPL.
 *
 *    See the LICENSE.lgpl and LICENSE.xpl files for the specific terms of 
 *    the licenses.
 *
 *    This software is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the README
 *    file for more details.
 *
 */
/*
 * Written by Tuomas Lukka
 */
package org.gzigzag.module;
import org.gzigzag.*;
import java.util.*;

/** A library of routines for operating on Nile streams.
 * @see Nile1
 */

public abstract class Nile2Ops {
public static final String rcsid = "$Id: Nile2Ops.java,v 1.3 2000/12/10 14:56:07 bfallenstein Exp $";
    public static boolean dbg = true;
    static void p(String s) { if(dbg) System.out.println(s); }
    static void pa(String s) { System.out.println(s); }

    // static Nile2Unit unit = new Nile2Unit.Sentence();

    static public void saveCursor(ZZCell view) {
	ZZCell s = view.getOrNewCell("d.nile-save", 1);
	if(ZZCursorReal.get(s) == null)
	    ZZCursorReal.set(s, new ZZCursorReal(view));
    }
    static public void clearSavedCursor(ZZCell view) {
	ZZCell s = view.getOrNewCell("d.nile-save", 1);
	if(ZZCursorReal.get(s) != null)
	    ZZCursorReal.set(s, (ZZCell)null);
    }
    static public ZZCursor getSavedCursor(ZZCell view) {
	ZZCell s = view.getOrNewCell("d.nile-save", 1);
	return new ZZCursorReal(s);
    }

    static public void setCursor(Nile2Unit unit, ZZCell view, ZZCursor to) {
	p("Set cursor: "+unit+" "+view+" "+to);
	if(to == null) {
	    // Saves only when there is nothing there before.
	    saveCursor(view);
	    to = getSavedCursor(view);
	} else {
	    clearSavedCursor(view);
	}
	Nile2Iter iter = giter(to);
	unit.start(iter, -1, true);
	curs(view).set(iter.getCursor());
	unit.end(iter, 1, true);
	endcurs(view).set(iter.getCursor());
    }

    static public void moveCursor(Nile2Unit unit, ZZCell view, int dir) {
	clearSavedCursor(view);
	Nile2Iter iter;
	if(dir > 0) {
	    iter = giter(endcurs(view));
	    if(!unit.insertPos(iter, 1, true)) return;
	} else {
	    iter = giter(curs(view));
	    if(!unit.insertPos(iter, -1, false)) return;
	}
	curs(view).set(iter.getCursor());
	unit.end(iter, 1, false);
	endcurs(view).set(iter.getCursor());
    }

    static public void adjust(Nile2Unit unit, ZZCell view, int end, int dir) {
	clearSavedCursor(view);
	Nile2Iter iter = giter(curs(view));
	Nile2Iter enditer = giter(endcurs(view));
	if(end > 0) {
	    if(!unit.end(enditer, dir, false)) return;
	} else {
	    if(!unit.start(iter, dir, false)) return;
	}
	if(!iter.isOrdered(enditer))
	    return;
	curs(view).set(iter.getCursor());
	endcurs(view).set(enditer.getCursor());
    }

    static public void hop(Nile2Unit unit, ZZCell view, int dir) {
	clearSavedCursor(view);
	Nile2Iter iter = giter(curs(view));
	Nile2Iter enditer = giter(endcurs(view));
	// XXX Checkpoint for undo!
	Nile2Iter pt = unit.cut(iter, enditer);

	unit.insertPos(pt, dir, false);

	unit.paste(pt, iter, enditer);

	curs(view).set(iter.getCursor());
	endcurs(view).set(enditer.getCursor());
    }

    /** Tunnel text from one nile window to another. */
    static public void tunnel(Nile2Unit unit, ZZCell from, ZZCell to, int dir) {
	clearSavedCursor(from); clearSavedCursor(to);
	Nile2Iter fiter = giter(curs(from));
	Nile2Iter fenditer = giter(endcurs(from));
	Nile2Iter titer = giter(curs(to));
	
	// XXX Checkpoint for undo!
	Nile2Iter pt = unit.cut(fiter, fenditer);
	
	unit.insertPos(titer, dir, true);
	
	unit.paste(titer, fiter, fenditer);
	
	curs(to).set(fiter.getCursor());
	endcurs(to).set(fenditer.getCursor());
	
	unit.start(pt, -1, false);
	curs(from).set(pt.getCursor());
	unit.end(pt, 1, true);
	endcurs(from).set(pt.getCursor());
    }

    static public boolean insert(ZZCell view, String key) {
	clearSavedCursor(view);
	if(key == null)
	    return false;
	if(key.length() != 1)
	    return false;
	Nile2Iter it = giter(curs(view));
	it.insert(key);
	ZZCursor c = it.getCursor();
	curs(view).set(c);
	endcurs(view).set(c);
	return true;
    }

    static public void backspace(Nile2Unit unit, ZZCell view) {
	clearSavedCursor(view);
	Nile2Iter iter = giter(curs(view));
	if(!unit.insertPos(iter, -1, false)) return;
	Nile2Iter enditer = (Nile2Iter)iter.clone();
	if(!unit.end(enditer, 1, false)) return;
	Nile2Iter cut = unit.cut(iter, enditer);
	if(cut == null)
	    throw new ZZError("Couldn't cut!");
	setCursor(unit, view, cut.getCursor());
    }

    static public void del(Nile2Unit unit, ZZCell view) {
	unit.cut(giter(curs(view)), giter(endcurs(view)));
    }

    static public void breakParagraph(Nile2Unit unit, ZZCell view) {
	clearSavedCursor(view);
	Nile2Iter it = giter(curs(view));
	it.breakParagraph();
	setCursor(unit, view, it.getCursor());
    }




    // Shorthand...
    static public ZZCursorReal curs(ZZCell view) {
	return new ZZCursorReal(view);
    }
    static public Nile2Iter giter(ZZCursor c) {
	return new Nile2Iter(c);
    }

    static public ZZCursorReal endcurs(ZZCell view) {
	ZZCell end = view.getOrNewCell("d.nile-sel");
	return new ZZCursorReal(end);
    }

}
