/*   
ZZPlayer.java
 *    
 *    Copyright (c) 2000, Ted Nelson and Tuomas Lukka
 *
 *    You may use and distribute under the terms of either the GNU Lesser
 *    General Public License, either version 2 of the license or,
 *    at your choice, any later version. Alternatively, you may use and
 *    distribute under the terms of the XPL.
 *
 *    See the LICENSE.lgpl and LICENSE.xpl files for the specific terms of 
 *    the licenses.
 *
 *    This software is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the README
 *    file for more details.
 *
 */
/*
 * Written by Tuomas Lukka
 */
package org.gzigzag.module;
import org.gzigzag.*;
import java.util.*;
import java.io.*;
import java.net.*;
import javax.media.*;

/** A player object, internally - no view. 
 */

public class ZZPlayer implements ControllerListener {
public static final String rcsid = "$Id: ZZPlayer.java,v 1.5 2000/10/18 14:35:32 tjl Exp $";
    public static final boolean dbg = true;
    static final void p(String s) { if(dbg) System.out.println(s); }
    static final void pa(String s) { System.out.println(s); }

    ZZPlayer() {
	t.start();
    }

    static Span cs(String stream, long o1, long o2) {
	return Span.create(
	    Address.streamOffs(stream, o1),
	    Address.streamOffs(stream, o2)
	);
    }

    static void crinst(ZZCell incur, ZZCell view, ZZView zzcv) {
    	if(instance!=null) return;
	ZZSoundCanvasComb scc = new ZZSoundCanvasComb(incur, view, zzcv);
	instance = new ZZPlayer();
	instance.mainCell = scc.getSoundViewCell();
    }

	static ZZPlayer instance;

	static public ZZModule module = new ZZModule() {
		public void action(String id,
			ZZCell code, ZZCell viewCursor, ZZCell view,
			ZZCell client, ZZCell clicked,
			ZZView zzv, ZZView zzcv) {
			p("JMFSoundScroll ACTION!");
			if(id.equals("CRDEMO")) {
				ZZCell c = viewCursor.getSpace().getHomeCell();
				c = c.N("d.2", -1);
				c.setText("Scrolls");
				c = c.N("d.1", 1);
				c.setText("s2");
				c = c.N("d.2", 1);
				c.setText("/BIG/demosound");
				c = c.N("d.2", 1);
				c.setText("Sound");
				c = c.N("d.2", 1);
				c.setText("20000000000");
				c = viewCursor.getSpace().getHomeCell();
				c = c.N("d.1", 1);
				c = c.N("d.1", 1);
				c = c.N("d.1", 1);
				ZZCell incur = c;
				for(int i=0; i<5; i++) {
				    c.setSpan(cs("s2", (2*i)*(long)1000000000, (2*i+1)*(long)1000000000));
				    c = c.N("d.2", 1);
				}
				c.delete();

				crinst(incur, view, zzcv);
			} else if(id.equals("PLAY")) {
			    crinst(viewCursor, view, zzcv);
			    instance.setLoop(false);
			    instance.play();
			} else if(id.equals("PLAYLOOP")) {
			    crinst(viewCursor, view, zzcv);
			    instance.setLoop(true);
			    instance.play();
			} else if(id.equals("PAUSE")) {
			    crinst(viewCursor, view, zzcv);
			    instance.setLoop(false);
			    instance.pause();
			} else if(id.equals("STOP")) {
			    crinst(viewCursor, view, zzcv);
			    instance.setLoop(false);
			    instance.stop();
			} else if(id.equals("TEST1")) {
				// p("Start test");
				// Span[] sps = new Span[] {
				// 	Span.parse("0.1000000000:0.2000000000"),
				// 	Span.parse("0.3000000000:0.4000000000"),
				// 	Span.parse("0.5000000000:0.6000000000"),
				// 	Span.parse("0.7000000000:0.8000000000"),
				// };
				// JMFSoundScroll s = new JMFSoundScroll("0",
				// 	new File("/BIG/demosound"));
				// s.play(sps);
			} else {
				pa("UNKNOWN ZZMBOX COMMAND "+id);
			}
		}
	};


    ZZCell mainCell;

    ZZCell lastCursor;
    SoundScroll curscroll;

    Player player;
    boolean playing;

    public void controllerUpdate(ControllerEvent ev) {
        p("PLAYER EV: " + ev + " ");
	if(ev instanceof StopAtTimeEvent) {
	    boolean loop = (mainCell.s("d.loop",1)!=null);
	    if(!loop) {
		ZZCell curs = getCursor();
		ZZCell n = curs.s("d.2", 1);
		if(n!=null)
			setCursor(n);
		else {
			stop();
			return;
		}
	    }
	    play_at_cursor();
	} 
    }
    protected void timedCheck() {
	p("PLAYER TCHECK");
	if(getCursor() != lastCursor && playing) {
	    p("MOVED CURSOR");
	    play_at_cursor();
	}
    }
    Thread t = new Thread() {
	public void run() {
	    while(true) {
		p("ZZPThread");
		try {
		    Thread.sleep(1000);
		    timedCheck();
		} catch(Exception e) {
		    pa(""+e);
		    e.printStackTrace();
		}
	    }
	}
    };

    protected void play_at_cursor() {
	ZZCell c = getCursor();
	Span s = c.getSpan();
	if(s==null) {
	    p("NULLSPAN");
	    return;
	}
	SoundScroll scr = (SoundScroll)s.getStart().getScroll(mainCell.getSpace());
	if(scr != curscroll) {
	    curscroll = scr;
	    URL url = null;
	    try {
		File f = curscroll.getFile();
		url = new URL("file", "", f.getCanonicalPath());
		player = Manager.createRealizedPlayer(url);
	    } catch(Exception e) {
		pa(""+e);
		e.printStackTrace();
	    }
	    player.addControllerListener(this);
	}
	player.stop();
	player.setMediaTime(new Time(s.getStart().getOffs()));
	player.setStopTime(new Time(s.getEnd().getOffs()));
	player.start();
	lastCursor = c;
    }
    
    public void play() {
	if(player == null || player.getTargetState() != player.Started) {
	    play_at_cursor();
	}
	playing = true;
    }
    public void stop() {
	player.stop();
	setCursor(getCursor().h("d.2",-1));
	playing = false;
    }
    public void pause() {
	player.stop();
	playing = false;
    }
    public void setCursor(ZZCell c) {
	c.insert("d.cursor", -1, mainCell.h("d.mycursor",1));
	
    }
    public ZZCell getCursor() {
	return mainCell.h("d.mycursor", 1).h("d.cursor", 1);
    }
    // XXX Maybe have the cells to be looping on as cursors on d.loop?
    public void setLoop(boolean is) { 
	    if(!is)
		    mainCell.disconnect("d.loop", 1);
	    else
		    mainCell.N("d.loop", 1);
    }
}



